@@ -1,12 +1,24 @@
 
-require 5; # -*-Text-*- Time-stamp: "2001-03-10 21:16:08 MST"
+require 5; # -*-Text-*- Time-stamp: "2002-08-16 11:53:47 MDT"
 package HTML::Tree;
-$VERSION = $VERSION = 3.10;
+$VERSION = $VERSION = 3.13;
   # This is where the dist gets its version from.
 
-# Basically a happy alias to HTML::TreeBuilder
+# Basically just a happy alias to HTML::TreeBuilder
 use HTML::TreeBuilder ();
-sub new { shift; HTML::TreeBuilder->new(@_); }
+
+sub new {
+  shift; unshift @_, 'HTML::TreeBuilder';
+  goto &HTML::TreeBuilder::new;
+}
+sub new_from_file {
+  shift; unshift @_, 'HTML::TreeBuilder';
+  goto &HTML::TreeBuilder::new_from_file;
+}
+sub new_from_content {
+  shift; unshift @_, 'HTML::TreeBuilder';
+  goto &HTML::TreeBuilder::new_from_content;
+}
 
 1;  
 
@@ -14,7 +26,7 @@ __END__
 
 =head1 NAME
 
-HTML-Tree - overview of HTML::TreeBuilder et al
+HTML::Tree - overview of HTML::TreeBuilder et al
 
 =head1 SYNOPSIS
 
@@ -73,9 +85,18 @@ L<HTML::Parser>
 
 L<HTML::DOMbo>
 
+The book I<Perl & LWP> by me, Sean M. Burke, published by
+O'Reilly and Associates, 2002.  ISBN: 0-596-00178-9
+
+It has several chapters to do with HTML processing in general,
+and HTML-Tree specifically.  There's more info at:
+
+  http://www.oreilly.com/catalog/perllwp/
+  http://www.amazon.com/exec/obidos/ASIN/0596001789
+
 =head1 COPYRIGHT
 
-Copyright 1995-1998 Gisle Aas; copyright 1999-2001 Sean M. Burke.
+Copyright 1995-1998 Gisle Aas; copyright 1999-2002 Sean M. Burke.
 (Except the articles contained in HTML::Tree::AboutObjects,
 HTML::Tree::AboutTrees, and HTML::Tree::Scanning, which are all
 copyright 2000 The Perl Journal.)
