@@ -1,6 +1,6 @@
 
 require 5;
-# Time-stamp: "2001-03-10 22:56:34 MST"
+# Time-stamp: "2002-08-16 11:49:28 MDT"
 package HTML::TreeBuilder;
 #TODO: maybe have it recognize higher versions of
 # Parser, and register the methods as subs?
@@ -11,8 +11,9 @@ package HTML::TreeBuilder;
 
 use strict;
 use integer; # vroom vroom!
+use Carp ();
 use vars qw(@ISA $VERSION $DEBUG);
-$VERSION = '3.10';
+$VERSION = '3.13';
 
 # TODO: thank whoever pointed out the TEXTAREA bug
 # TODO: make require Parser of at least version... 2.27?
@@ -80,13 +81,60 @@ use HTML::Parser ();
 *HTML::TreeBuilder::p_closure_barriers = \@HTML::Tagset::p_closure_barriers;
 
 #==========================================================================
+# Two little shortcut constructors:
+
+sub new_from_file { # or from a FH
+  my $class = shift;
+  Carp::croak("new_from_file takes only one argument")
+   unless @_ == 1;
+  Carp::croak("new_from_file is a class method only")
+   if ref $class;
+  my $new = $class->new();
+  $new->parse_file($_[0]);
+  return $new;
+}
+
+sub new_from_content { # from any number of scalars
+  my $class = shift;
+  Carp::croak("new_from_content is a class method only")
+   if ref $class;
+  my $new = $class->new();
+  foreach my $whunk (@_) {
+    if(ref($whunk) eq 'SCALAR') {
+      $new->parse($$whunk);
+    } else {
+      $new->parse($whunk);
+    }
+    last if $new->{'_stunted'}; # might as well check that.
+  }
+  $new->eof();
+  return $new;
+}
+
+# TODO: document?
+sub parse_content {  # from any number of scalars
+  my $tree = shift;
+  my $retval;
+  foreach my $whunk (@_) {
+    if(ref($whunk) eq 'SCALAR') {
+      $retval = $tree->parse($$whunk);
+    } else {
+      $retval = $tree->parse($whunk);
+    }
+    last if $tree->{'_stunted'}; # might as well check that.
+  }
+  $tree->eof();
+  return $retval;
+}
+
+
+#---------------------------------------------------------------------------
 
 sub new { # constructor!
   my $class = shift;
   $class = ref($class) || $class;
 
   my $self = HTML::Element->new('html');  # Initialize HTML::Element part
-
   {
     # A hack for certain strange versions of Parser:
     my $other_self = HTML::Parser->new();
@@ -108,6 +156,8 @@ sub new { # constructor!
   $self->{'_tighten'} = 1;
     # whether ignorable WS in this tree should be deleted
 
+  $self->{'_implicit'} = 1;  # to delete, once we find a real open-"html" tag
+
   $self->{'_element_class'}      = 'HTML::Element';
   $self->{'_ignore_unknown'}     = 1;
   $self->{'_ignore_text'}        = 0;
@@ -193,6 +243,14 @@ sub warning {
       # inserted by some lame code-generators.
       return;    # bypass tweaking.
     }
+   
+    $tag =~ s{/$}{}s;  # So <b/> turns into <b>.  Silently forgive.
+    
+    unless($tag =~ m/^[-_a-zA-Z0-9:%]+$/s) {
+      DEBUG and print "Start-tag name $tag is no good.  Skipping.\n";
+      return;
+      # This avoids having Element's new() throw an exception.
+    }
 
     my $ptag = (
                 my $pos  = $self->{'_pos'} || $self
@@ -212,6 +270,15 @@ sub warning {
     }
     
     #print $indent, "POS: $pos ($ptag)\n" if DEBUG > 2;
+    # $attr = {%$attr};
+
+    foreach my $k (keys %$attr) {
+      # Make sure some stooge doesn't have "<span _content='pie'>".
+      # That happens every few million Web pages.
+      $attr->{' ' . $k} = delete $attr->{$k}
+       if length $k and substr($k,0,1) eq '_';
+      # Looks bad, but is fine for round-tripping.
+    }
     
     my $e =
      ($self->{'_element_class'} || 'HTML::Element')->new($tag, %$attr);
@@ -427,6 +494,12 @@ sub warning {
                 }
             }
         } elsif ($HTML::TreeBuilder::isTableElement{$tag}) {
+            if(!$pos->is_inside('table')) {
+                print $indent, " * \U$tag\E makes an implicit TABLE\n"
+                  if DEBUG > 1;
+                $self->insert_element('table', 1);
+            }
+
             if($tag eq 'td' or $tag eq 'th') {
                 # Get under a tr one way or another
                 unless(
@@ -455,11 +528,6 @@ sub warning {
             #   th closes any open th OR td (limited at a table)
             #   ...implementable as "close to a tr, or make a tr"
             
-            if(!$pos->is_inside('table')) {
-                print $indent, " * \U$tag\E makes an implicit TABLE\n"
-                  if DEBUG > 1;
-                $self->insert_element('table', 1);
-            }
         } elsif ($HTML::TreeBuilder::isPhraseMarkup{$tag}) {
             if($ptag eq 'body' and $self->{'_implicit_body_p_tag'}) {
                 print
@@ -551,11 +619,18 @@ sub warning {
           # otherwise it'll be under the noframes already
         and !$self->is_inside('body')
       ) {
+	# The following is a bit of a hack.  We don't use the normal
+        #  insert_element because 1) we don't want it as _pos, but instead
+        #  right under $self, and 2), more importantly, that we don't want
+        #  this inserted at the /end/ of $self's content_list, but instead
+        #  in the middle of it, specifiaclly right before the body element.
+        #
         my $c = $self->{'_content'} || die "Contentless root?";
         my $body = $self->{'_body'} || die "Where'd my BODY go?";
         for(my $i = 0; $i < @$c; ++$i) {
           if($c->[$i] eq $body) {
             splice(@$c, $i, 0, $self->{'_pos'} = $pos = $e);
+	    $e->{'_parent'} = $self;
             $already_inserted = 1;
             print $indent, " * inserting 'frameset' right before BODY.\n"
              if DEBUG > 1;
@@ -697,6 +772,12 @@ sub warning {
       return;
     }
 
+    unless(ref($tag) or $tag =~ m/^[-_a-zA-Z0-9:%]+$/s) {
+      DEBUG and print "End-tag name $tag is no good.  Skipping.\n";
+      return;
+      # This avoids having Element's new() throw an exception.
+    }
+
     # This method accepts two calling formats:
     #  1) from Parser:  $self->end('tag_name', 'origtext')
     #        in which case we shouldn't mistake origtext as a blocker tag
@@ -1194,6 +1275,9 @@ sub process {
 
 sub eof {
   # Accept an "end-of-file" signal from HTML::Parser, or thrown by the user.
+  
+  return if $_[0]->{'_done'}; # we've already been here
+  
   return $_[0]->SUPER::eof() if $_[0]->{'_stunted'};
   
   my $x = $_[0];
@@ -1230,7 +1314,8 @@ sub eof {
   $x->delete_ignorable_whitespace()
    # this's why we trap this -- an after-method
    if $x->{'_tighten'} and ! $x->{'_ignore_text'};
-
+  $x->{'_done'} = 1;
+  
   return @rv if wantarray;
   return $rv[0];
 }
@@ -1242,6 +1327,7 @@ sub eof {
 sub stunt {
   my $self = $_[0];
   print "Stunting the tree.\n" if DEBUG;
+  $self->{'_done'} = 1;
   
   if($HTML::Parser::VERSION < 3) {
     #This is a MEAN MEAN HACK.  And it works most of the time!
@@ -1271,6 +1357,7 @@ sub stunt {
 
 # TODO: document
 sub stunted  { shift->_elem('_stunted',  @_); }
+sub done     { shift->_elem('_done',     @_); }
 
 #==========================================================================
 
@@ -1307,6 +1394,10 @@ sub delete {
      if defined $_ and ref $_   #  Make sure it's an object.
         and $_ ne $_[0];   #  And avoid hitting myself, just in case!
   }
+
+  $_[0]->detach if $_[0]->{'_parent'} and $_[0]->{'_parent'}{'_content'};
+   # An 'html' element having a parent is quite unlikely.
+
   return undef;
 }
 
@@ -1325,9 +1416,47 @@ sub elementify {
     and $_ ne '_implicit' and $_ ne '_pos'
     and $_ ne '_element_class'
   } keys %$self };
-  bless $self, $to_class;
+  bless $self, $to_class;   # Returns the same object we were fed
 }
 
+#--------------------------------------------------------------------------
+
+sub guts {
+  my @out;
+  my @stack = ($_[0]);
+  my $destructive = $_[1];
+  my $this;
+  while(@stack) {
+    $this = shift @stack;
+    if(!ref $this) {
+      push @out, $this;  # yes, it can include text nodes
+    } elsif(! $this->{'_implicit'}) {
+      push @out, $this;
+      delete $this->{'_parent'} if $destructive;
+    } else {
+      # it's an implicit node.  Delete it and recurse
+      delete $this->{'_parent'} if $destructive;
+      unshift @stack, @{
+        (  $destructive ? 
+           delete($this->{'_content'})
+           :      $this->{'_content'}
+        )
+        || []
+      };
+    }
+  }
+  # Doesn't call a real $root->delete on the (when implicit) root,
+  #  but I don't think it needs to.
+  
+  return @out if wantarray;  # one simple normal case.
+  return undef unless @out;
+  return $out[0] if @out == 1 and ref($out[0]);
+  my $x = HTML::Element->new('div', '_implicit' => 1);
+  $x->push_content(@out);
+  return $x;
+}
+
+sub disembowel { $_[0]->guts(1) }
 
 #--------------------------------------------------------------------------
 1;
@@ -1367,6 +1496,9 @@ $tree->parse($document_content) and $tree->eof if you've got
 the content in a string) to parse the HTML
 document into the tree $tree.
 
+(You can combine steps 1 and 2 with the "new_from_file" or
+"new_from_content" methods.)
+
 2b. call $root-E<gt>elementify() if you want.
 
 3. do whatever you need to do with the syntax tree, presumably
@@ -1389,7 +1521,7 @@ HTML::Element documentation, and also skim the HTML::Parser
 documentation -- probably only its parse and parse_file methods are of
 interest.
 
-The following methods native to HTML::TreeBuilder all control how
+Most of the following methods native to HTML::TreeBuilder control how
 parsing takes place; they should be set I<before> you try parsing into
 the given object.  You can set the attributes by passing a TRUE or
 FALSE value as argument.  E.g., $root->implicit_tags returns the current
@@ -1398,6 +1530,25 @@ option on, and $root->implicit_tags(0) turns it off.
 
 =over 4
 
+=item $root = HTML::TreeBuilder->new_from_file(...)
+
+This "shortcut" constructor merely combines constructing a new object
+(with the "new" method, below), and calling $new->parse_file(...) on
+it.  Returns the new object.  Note that this provides no way of
+setting any parse options like store_comments (for that, call new, and
+then set options, before calling parse_file).  See the notes (below)
+on parameters to parse_file.
+
+=item $root = HTML::TreeBuilder->new_from_content(...)
+
+This "shortcut" constructor merely combines constructing a new object
+(with the "new" method, below), and calling for(...){$new->parse($_)}
+and $new->eof on it.  Returns the new object.  Note that this provides
+no way of setting any parse options like store_comments (for that,
+call new, and then set options, before calling parse_file).  Example
+usages: HTML::TreeBuilder->new_from_content(@lines), or
+HTML::TreeBuilder->new_from_content($content)
+
 =item $root = HTML::TreeBuilder->new()
 
 This creates a new HTML::TreeBuilder object.  This method takes no
@@ -1406,7 +1557,11 @@ attributes.
 =item $root->parse_file(...)
 
 [An important method inherited from L<HTML::Parser|HTML::Parser>, which
-see.]
+see.  Current versions of HTML::Parser can take a filespec, or a
+filehandle object, like *FOO, or some object from class IO::Handle,
+IO::File, IO::Socket) or the like.
+I think you should check that a given file exists I<before> calling 
+$root->parse_file($filespec).]
 
 =item $root->parse(...)
 
@@ -1432,10 +1587,11 @@ see.]
 
 This changes the class of the object in $root from
 HTML::TreeBuilder to the class used for all the rest of the elements
-in that tree (generally HTML::Element).
+in that tree (generally HTML::Element).  Returns $root.
 
 For most purposes, this is unnecessary, but if you call this after
-you're finished building a tree, then it keeps you from accidentally
+(after!!)
+you've finished building a tree, then it keeps you from accidentally
 trying to call anything but HTML::Element methods on it.  (I.e., if
 you accidentally call C<$root-E<gt>parse_file(...)> on the
 already-complete and elementified tree, then instead of charging ahead
@@ -1448,6 +1604,52 @@ $root except for "_tag", "_parent", "_content", "_pos", and
 "_implicit".  If anyone requests that I change this to leave in yet
 more private attributes, I might do so, in future versions.
 
+=item @nodes = $root->guts()
+
+=item $parent_for_nodes = $root->guts()
+
+In list context (as in the first case), this method returns the topmost
+non-implicit nodes in a tree.  This is useful when you're parsing HTML
+code that you know doesn't expect an HTML document, but instead just
+a fragment of an HTML document.  For example, if you wanted the parse
+tree for a file consisting of just this:
+
+  <li>I like pie!
+
+Then you would get that with C<< @nodes = $root->guts(); >>.
+It so happens that in this case, @notes will contain just one
+element object, representing the "li" node (with "I like pie!" being
+its text child node).  However, consider if you were parsing this:
+
+  <hr>Hooboy!<hr>
+
+In that case, C<< $root->guts() >> would return three items:
+an element object for the first "hr", a text string "Hooboy!", and
+another "hr" element object.
+
+For cases where you want definitely one element (so you can treat it as
+a "document fragment", roughly speaking), call C<guts()> in scalar
+context, as in C<< $parent_for_nodes = $root->guts() >>. That works like
+C<guts()> in list context; in fact, C<guts()> in list context would
+have returned exactly one value, and if it would have been an object (as
+opposed to a text string), then that's what C<guts> in scalar context
+will return.  Otherwise, if C<guts()> in list context would have returned
+no values at all, then C<guts()> in scalar context returns undef.  In
+all other cases, C<guts()> in scalar context returns an implicit 'div'
+element node, with children consisting of whatever nodes C<guts()>
+in list context would have returned.  Note that that may detach those
+nodes from C<$root>'s tree.
+
+=item @nodes = $root->disembowel()
+
+=item $parent_for_nodes = $root->disembowel()
+
+The C<disembowel()> method works just like the C<guts()> method, except
+that disembowel definitively destroys the tree above the nodes that
+are returned.  Usually when you want the guts from a tree, you're just
+going to toss out the rest of the tree anyway, so this saves you the
+bother.  (Remember, "disembowel" means "remove the guts from".)
+
 =item $root->implicit_tags(value)
 
 Setting this attribute to true will instruct the parser to try to
@@ -1670,7 +1872,7 @@ L<HTML::DOMbo>
 
 =head1 COPYRIGHT
 
-Copyright 1995-1998 Gisle Aas; copyright 1999-2001 Sean M. Burke.
+Copyright 1995-1998 Gisle Aas; copyright 1999-2002 Sean M. Burke.
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
